;
; This file is released under the terms of the GNU General Public License
; as they appear in http://www.gnu.org/licenses/gpl.txt.
;
; Copyright 2009-2017 erwin.rieger@ibrieger.de
;
;
; Programm fuer Pic12f629, Funk-Lichtschranke
;
;
; 12F629 wird mit 4Mhz (intern) betrieben
;
;
; Pic begibt sich nach dem init in den sleep-modus. Durch fallende flanke
; an GP2 wachen wir auf und versenden 200 datagramme an GP0 mit:
;
;   * Sync-Sequenz, 8 bit
;   * Startbit
;   * 8bit - Zaehler
;   * 8bit - ID + Pruefsumme
;
; Danach gehen wir wieder in den sleep-modus.
;

	LIST	p=PIC12F629, f=INHX16
	include "p12f629.inc"
    ERRORLEVEL      -302            ;SUPPRESS BANK SELECTION MESSAGES

; Configuration festlegen, Power up Timer, kein Watchdog, int-Oscillator, kein Brown out
    __CONFIG   _MCLRE_OFF & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT & _BODEN_OFF & _CP_OFF & _CPD_OFF

; Variablen festlegen  20h ... 5Fh 
packetCounter   equ     0x22
counter         equ     0x23
delayCount      equ     0x24
bitCount        equ     0x25
outByte         equ     0x26            ; Zwischenspeicher ausgabebyte
pruefSumme      equ     0x27            ; High-nibble: ModuleId, low-nibble: pruefsumme

MODULEID        equ     0x20            ; Lichtschranken-ID

;***************************************************************************************

	org 0
	goto main

;***************************************************************************************

main:

    ; INIT:
    ; IO-Pins
    bcf     STATUS, RP0             ; Bank 0
    clrf    GPIO                    ; aus!
    movlw   0x07
    movwf   CMCON                   ; alle Pins digital (nicht Comp)

    bsf     STATUS, RP0             ; Bank 1
    ; set GPIO2, 3 as input, rest as output (0=output, 1=input):
    movlw   b'00001100'
    movwf   TRISIO

    ; internen Taktgenerator kalibrieren
    call    0x3FF
    movwf   OSCCAL

    ; Interrupt
    bcf     INTCON, GIE             ; Int deaktiviert
    bsf     INTCON, INTE            ; Enable GP2/INT
    bcf     OPTION_REG, INTEDG      ; Falling edge
    bcf     OPTION_REG, NOT_GPPU    ; Enable pull-ups on inputs
    ; bsf     WPU, WPU2               ; Enable pull-up on GPIO2

    bcf     STATUS, RP0  ; Bank 0

;***************************************************************************************

MainLoop:               ; Hauptschleife

	movlw 0xc8		    ; packet count, 200
	movwf packetCounter	; packet counter

    bcf INTCON,INTF     ; Clear interruptflag

    sleep               ; wakeup durch positive flanke an gpio2

PacketLoop:             ; Ausgabe ein datagramm

    movlw MODULEID
    movwf pruefSumme    ; pruefSumme vorbelegen / zuruecksetzen

    ;
    ; Sync-sequenz ausgeben, 2000Hz
    ;
	movlw 0x7 		    ; sync-count
	movwf counter

Sync:                   ; Ausgabe sync-sequenz
    bsf GPIO, 0         ; Gpio 0 high
    call Wait250
    bcf GPIO, 0         ; Gpio 0 low
    call Wait250

    decfsz counter, 1
    goto Sync

    ;
    ; Startbit ausgeben, 750uS high, 250uS low
    ;
    bsf GPIO, 0         ; Gpio 0 high
    call Wait250
    call Wait250
    call Wait250
    bcf GPIO, 0         ; Gpio 0 low
    call Wait250

    ;
    ; Packet-Counter ausgeben, 2000Hz
    ;
    movf packetCounter, 0 ; packetCounter -> W
    movwf outByte
    call mOut           ; output manchester code

    ;
    ; Module-ID + Pruefsumme ausgeben, 2000Hz
    ;
    movf pruefSumme, 0  ; pruefSumme -> W
    movwf outByte
    call mOut           ; output manchester code

    decfsz packetCounter, 1
    goto PacketLoop

    bcf GPIO, 0         ; Gpio 0 low

	goto MainLoop

;***************************************************************************************

;
; call:     2i
; init:     2i
; schleife: 3i*n
; ret:      2i
; t = 6i + 3i*n
; nt = (t - 6) / 3
;
Wait250:                ; Wait 250 uS
    movlw 0x51          ; 1i
    movwf delayCount    ; 1i
w250:
    decfsz delayCount, 1 ; 1i bis auf's letzte mal
    goto w250           ; 2i
    return              ; 2i

;
; Output byte in outByte, manchester encoded, at 2000Hz
; logic 1 -> 01 wechsel
; logic 0 -> 10 wechsel
;
mOut:
    movlw 0x8           ; output 8 bits
    movwf bitCount

bitOut:
    btfss outByte, 7
    goto zeroOut

; oneOut:               ; output logic one
    incf pruefSumme, 1  ; pruefSumme++
    bcf GPIO, 0         ; Gpio 0 low
    call Wait250
    bsf GPIO, 0         ; Gpio 0 high
    call Wait250
    goto nextBit

zeroOut:                ; output logic zero
    bsf GPIO, 0         ; Gpio 0 high
    call Wait250
    bcf GPIO, 0         ; Gpio 0 low
    call Wait250

nextBit:
    rlf outByte, 1      ; shift left, store result in outByte
    decfsz bitCount, 1
    goto bitOut

    return

;OSCCAL calibration value
; org   0x3ff
; retlw 0x44

	end



